%% Prepare data for Matlab
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% requires Mapping Toolbox in MATLAB
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

close all; clear all;

datadir = 'Bay Area Data Files\';
picdir  = 'figures\';

%% Read map data

zip2     = shaperead('Shape\Full_Bay_Area_Zip_Codes_S');
water    = shaperead('Shape\bayarea_majorwater');
counties = shaperead('Shape\bayareacounty00'); % not water clipped

% Remove Napa county - not part of our definition of Bay Area
counties = counties([1:4 6:end]);

% Create latitude & longitude of zip code centroids
%  centroids array will have 3 columns:
%    1. zipcode, 2. latitude, 3. longitude

centroids = zeros(length(zip2),3);

for zz=1:length(zip2)
    zipnow = str2num(zip2(zz).ZCTA5CE10);
    zip2(zz).Lat = zip2(zz).Y;
    zip2(zz).Lon = zip2(zz).X;
    centroids2(zz,:) = [zipnow, str2num(zip2(zz).INTPTLAT10), str2num(zip2(zz).INTPTLON10)];
end

% Order in ascending order of zipcode, as is the case for segments
centroids=sortrows(centroids2,1);

% Split into make column vectors:
zipgeo     = centroids(:,1);  % list of zip-codes for which we have centroids
lat        = centroids(:,2);
lon        = centroids(:,3);

  
%% Read Bay Area Housing Data (all comma-delimited)

%  Variables used in definition + stocks
a    = importdata([datadir 'segment_def.txt'],',',1);

% Segment averages for volume, TOM etc
b    = importdata([datadir 'segments.txt'],',',1);

% Same as b for forclosures
c    = importdata([datadir 'segments_FC_REO.txt'],',',1);

% Strings with city names
fid  = fopen([datadir 'city_names.txt']);
d    = textscan(fid,'%q');
fclose(fid);

%  Relates zips to their cities
e = importdata([datadir 'city_zips.txt'],',',1);

% Drop if no geo info: keepseg will be carried forward to format other data sets also
keepseg = ismember(a.data(:,1),zipgeo); 

% Segments
seg_zips        = a.data(:,1);
seg_stocks      = a.data(:,5);
seg_stocks_acs  = a.data(:,6);
seg_def         = a.data(:,2:4);

% Zip codes 
zips     = unique(seg_zips);
zipcode  = zips; % Legacy name
num_zips = length(zips);
num_segs = length(seg_zips);
segs     = [1:num_segs]';

% Cities
cityzipmatrix = e.data;
citynames     = d{1}(2:end);
zip_cities    = zeros(num_zips,1);
seg_cities    = zeros(num_segs,1);

% Build matrix aggcity to aggregate from segments to cities
aggcity       = zeros(size(cityzipmatrix,1),num_segs);

count = 1;
keep=[];
for i=1:size(cityzipmatrix,1)
    zipsincity = cityzipmatrix(i,~isnan(cityzipmatrix(i,:)));
    [vv,ia,ib]=intersect(zipsincity,zips);
    if ~isempty(vv)
         zip_cities(ib)=count;
         zip_citiesN(ib)=citynames(i);
         for j=1:length(zipsincity)                 
           seg_cities(seg_zips==zipsincity(j))=count;
           seg_citiesN(seg_zips==zipsincity(j))=citynames(i);
           aggcity(i,seg_zips==zipsincity(j))=1;
         end                           
         %unique(seg_cities)'
         count=count+1;
         keep=[keep;i];         
    end
end

citynames  = citynames(keep);
num_cities = length(citynames);
aggcity    = aggcity(keep,:);

% Segment-specific averages: combine data on foreclosures & regular sales
% (first row is segment numbers - see variable headers below)

numbers    = b.data(:,2:end);
names      = b.colheaders;
numbers_fc = c.data(:,2:end);

% Weight by volume or add depnding on variable additive columns 
additive    = [1:3];

% Volweighted columns
volweighted = [6 8 10 12];

vol      = 3;
volw     = numbers(:,vol)./(numbers(:,vol) + numbers_fc(:,vol));
volw_vec = volw*ones(1,length(volweighted));

numbers_total                = zeros(size(numbers));
numbers_total(:,additive)    = numbers(:,additive) + numbers_fc(:,additive);
numbers_total(:,4)           = numbers(:,4);
numbers_total(:,volweighted) = volw_vec.*numbers(:,volweighted)+(1-volw_vec).*numbers_fc(:,volweighted);

apartment_share = numbers(:,end);

% Build matrix aggzip to aggregate from segment to zipcode

aggzip = zeros(num_zips,num_segs);
seg_zips_str = cell(num_segs);
for zz=1:num_zips
    aggzip(zz,:) = (seg_zips==zips(zz));
    seg_zips_str{zz}=num2str(zips(zz));
end

% Vectors with segment centroids for plotting arrange around zipcode centroids in a circle tiny version is for small plots
radius       = 0.01;
lat_seg      = zeros(num_segs,1); 
lon_seg      = zeros(num_segs,1); 
lat_seg_tiny = zeros(num_segs,1); 
lon_seg_tiny = zeros(num_segs,1); 

num_sub_segs=sum(aggzip');

for zz=1:num_zips
    ensemble                          = [0:2*pi/num_sub_segs(zz):2*pi*(1-1/num_sub_segs(zz))];
    lat_seg(seg_zips==zips(zz))       = lat(zz) + radius*cos(ensemble);
    lon_seg(seg_zips==zips(zz))       = lon(zz) + radius*sin(ensemble);
    lat_seg_tiny(seg_zips==zips(zz))  = lat(zz) + radius*cos(ensemble)/2;
    lon_seg_tiny(seg_zips==zips(zz))  = lon(zz) + radius*sin(ensemble)/2;
end

% secondThird dimension: variables (Choose those variables that we want picked out from the dataset - this can be a subset of the total variables
% available (as it is in this case, where we also have mean price, and a number of choices of how TOM is computed).

num_vars     = 10;   %
varnames{1}  = 'inventory_avg';
varnames{2}  = 'inventory_c_avg';
varnames{3}  = 'number_al';             % Volume over 4 years
varnames{4}  = 'segment_stock';         % Housing stock
varnames{5}  = 'volume_share';
varnames{6}  = 'share_hot_sales';
varnames{7}  = 'sp_med';                % Median price
varnames{8}  = 'sp_mean';               % Mean price
varnames{9}  = 'time_on_market_med';    % Median tom, DAYS
varnames{10} = 'time_on_market_mean';   % Mean TOM
varnames{11} = 'tom_med_ge30';          % Median tom, DAYS
varnames{12} = 'tom_mean_ge30';         % Mean TOM

% Make print format for variables
mainf    = '%3.1f  %3.1f %4d %7d %0.3f %0.3f %7d %7d %3d %3d %3d %3d';
form_all = ['  %3d  %5d   ' mainf ' \n'];

%% Reduce to tsmeans dataset used in our analysis

% (select vars, define monthly turnover & inventory share and restate tom in months
%  Note: here selecting medians for price and tom

%------ make print format for variables
varnames2{1} = 'turnover';
varnames2{2} = 'price';
varnames2{3} = 'time on market ge30';
varnames2{4} = 'inventory';
varnames2{5} = 'hot sales';
varnames2{6} = 'time on market';
mainf2       = '%1.3f  %7.0f %3.1f %1.3f %0.2f %3.1f';
form_all2    = ['  %3d  %5d   ' mainf2 ' \n'];

% clean relevant numbers: patch cross sectional mean
for ii=[1 3 4 6 7 9]
    bad = isnan(numbers(:,ii));
    disp([num2str(sum(bad)) 'NaNs in ' varnames{ii}]);
    avg = mean(numbers(~bad,ii));
    numbers(bad,ii)=avg;
end

% Columns are the 6 above in varnames2

% Entry 1: segments, armslength
tsmeans_array{1} = [numbers(:,3)./(48*numbers(:,4)), numbers(:,8)...
                    numbers(:,12)/30 numbers(:,1)./numbers(:,4) numbers(:,6) numbers(:,10)/30];
% Entry 2; segments, incl FC
tsmeans_array{2} = [numbers_total(:,3)./(48*numbers_total(:,4)), numbers_total(:,8),...
                    numbers_total(:,12)/30 numbers_total(:,1)./numbers_total(:,4) numbers_total(:,6)...
                    numbers_total(:,10)/30];

% Zip code aggregates
zipvol       = aggzip*numbers(:,3);
zipstocks    = aggzip*numbers(:,4);
zipvol_total = aggzip*numbers_total(:,3);

% City aggregates
cityvol       = aggcity*numbers(:,3);
citystocks    = aggcity*numbers(:,4);
cityvol_total = aggcity*numbers_total(:,3);

% Entry 3: zipcodes, armslength
tsmeans_array{3} = [zipvol./(48*zipstocks),...
                    (aggzip*(numbers(:,8).*numbers(:,3)))./zipvol,...
                    (aggzip*(numbers(:,12).*numbers(:,3)))./(30*zipvol),...
                    (aggzip*numbers(:,1))./zipstocks,...
                    (aggzip*(numbers(:,6).*numbers(:,3)))./zipvol,...
                    (aggzip*(numbers(:,10).*numbers(:,3)))./(30*zipvol)];

% Entry 4: zipcodes, incl FC
tsmeans_array{4} = [zipvol_total./(48*zipstocks),...
                    (aggzip*(numbers_total(:,8).*numbers_total(:,3)))./zipvol_total,...
                    (aggzip*(numbers_total(:,12).*numbers_total(:,3)))./(30*zipvol_total),...
                    (aggzip*numbers_total(:,1))./zipstocks,...
                    (aggzip*(numbers_total(:,6).*numbers(:,3)))./zipvol,...
                    (aggzip*(numbers_total(:,10).*numbers_total(:,3)))./(30*zipvol_total)];

% Entry 5: cities, armslength
tsmeans_array{5} = [cityvol./(48*citystocks),...
                    (aggcity*(numbers(:,8).*numbers(:,3)))./cityvol,...
                    (aggcity*(numbers(:,12).*numbers(:,3)))./(30*cityvol),...
                    (aggcity*numbers(:,1))./citystocks,...
                    (aggcity*(numbers(:,6).*numbers(:,3)))./cityvol,...
                    (aggcity*(numbers(:,10).*numbers(:,3)))./(30*cityvol)];

% Entry 6: cities, incl FC
tsmeans_array{6} = [cityvol_total./(48*citystocks),...
                    (aggcity*(numbers_total(:,8).*numbers_total(:,3)))./cityvol_total,...
                    (aggcity*(numbers_total(:,12).*numbers_total(:,3)))./(30*cityvol_total),...
                    (aggcity*numbers_total(:,1))./citystocks,...
                    (aggcity*(numbers_total(:,6).*numbers(:,3)))./cityvol,...
                    (aggcity*(numbers_total(:,10).*numbers_total(:,3)))./(30*cityvol_total)];

save inistuff;

